@@ -1,8 +1,8 @@
 # this is stuff that needs to be run first (and in the right order) for S4 crap to work
 
 methods::setOldClass("UERE")
-new.UERE <- methods::setClass("UERE",contains="matrix",representation=methods::representation(DOF="matrix",AICc="numeric"),
-                              prototype=methods::prototype(matrix(),DOF=matrix(),AICc=numeric()))
+new.UERE <- methods::setClass("UERE",contains="matrix",representation=methods::representation(DOF="matrix",AICc="numeric",Zsq="numeric"),
+                              prototype=methods::prototype(matrix(),DOF=matrix(),AICc=numeric(),Zsq=numeric()))
 
 methods::setOldClass("telemetry")
 new.telemetry <- methods::setClass("telemetry",contains="data.frame",representation=methods::representation(info="list",UERE="UERE"),
@@ -45,3 +45,4 @@ writeShapefile <- function(object,folder,file=NULL,...) UseMethod("writeShapefil
 emulate <- function(object,...) UseMethod("emulate")
 AICc <- function(object,...) UseMethod("AICc")
 speed <- function(object,...) UseMethod("speed")
+speeds <- function(object,...) UseMethod("speeds")
