@@ -345,3 +345,14 @@ PDclamp <- function(M)
 
   return(M)
 }
+
+# relatively smallest eigen value of matrix
+mat.min <- function(M)
+{
+  if(any(diag(M)==0)) { return(0) }
+  diag(M) <- abs(diag(M))
+  M <- stats::cov2cor(M)
+  M <- eigen(M,only.values=TRUE)$values
+  M <- last(M)
+  return(M)
+}
